/*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
// (C) PicoPeta Simputers Pvt. Ltd. (2003)
// Author: Vivek K S

#include "alchemy.h"
#include "dialog_window.h"
#include "basic_window.h"
#include <X11/Xlib.h>
#include <X11/Xutil.h>

// A new dialog begins its journey
window *new_dialog_window(wm *wm, Window map_window)
{
	window *win;
	XSetWindowAttributes attr;
	XWindowAttributes winattr;
	unsigned int dialogbar_height;
	ac_theme *theme = wm->current_theme;
	Window transient_win;

	// Allocate and assign common data structures
	win=new_basic_window(wm, map_window, DIALOG_WIN);

	// Set Function pointers
	win->redraw = &dialog_window_redraw;
	win->refresh_widgets = &dialog_window_refresh_widgets;
	win->set_size = &dialog_window_set_size;
	win->set_position = &dialog_window_set_position;
	win->show = &dialog_window_show;
	win->hide = &dialog_window_hide;
	win->iconify = &dialog_window_hide;
	win->destroy = &dialog_window_destroy;
	win->set_flags = &dialog_window_set_flags;
	win->set_flag = &dialog_window_set_flag;
	win->app_window = map_window;

	win->flags.all_flags = 0;
	win->flags.bits.close = 1; // Only one close button

	// Figure out topbar size
	dialogbar_height = theme->get_dialogbar_height(theme);

	// Get transient hint
	XGetTransientForHint(wm->display, win->app_window, &transient_win);
	if(transient_win && transient_win != win->app_window)
		win->transient_of =  ac_get_window(wm, transient_win);

	// Link into the list of transients
	if(win->transient_of) {
		win->transient_next = win->transient_of->transient_next;
		win->transient_prev = win->transient_of;
		win->transient_of->transient_next = win;
		if(win->transient_next)
			win->transient_next->transient_prev = win;
	}

	// Figure out window size
	XGetWindowAttributes(wm->display, win->app_window, &winattr);
	win->x = winattr.x;
	win->y = winattr.y;
	win->width = winattr.width;
	win->height = winattr.height;

	if(win->width > (wm->display_width - 20))
		win->width = wm->display_width - 20;
	if((win->height+dialogbar_height) > (wm->display_height - 30))
		win->height = wm->display_height - 30 - dialogbar_height;

	win->x = (wm->display_width - win->width)/2;
	win->y = (wm->display_height - (win->height + dialogbar_height))/2;

	// Reparenting happens here
	// Create toplevel and topbar frames
	attr.override_redirect = True;
	attr.border_pixel = attr.background_pixel = wm->black_pixel;
	attr.event_mask = SubstructureRedirectMask|SubstructureNotifyMask|
		ButtonPressMask|ButtonReleaseMask|ExposureMask;

	win->frame_window = XCreateWindow(wm->display, wm->root, win->x,
			win->y - dialogbar_height, win->width,
			win->height + dialogbar_height, 0,
			CopyFromParent, CopyFromParent, CopyFromParent,
			CWOverrideRedirect|CWBorderPixel|CWEventMask|CWBackPixel,
			&attr);

	// Draw the topbar
	theme->render_theme(theme, win);

	// Set the cursor
	XDefineCursor(win->wm->display, win->topbar_frame->win, win->wm->normal_cursor);

	// In case we exit, the window still stays
	XAddToSaveSet(wm->display, win->app_window);

	XSelectInput(wm->display, win->app_window,
		ColormapChangeMask|PropertyChangeMask);

	XReparentWindow(wm->display, win->app_window, win->frame_window,
			0, dialogbar_height);

	if((win->width != winattr.width) || (win->height != winattr.height)) {
		XResizeWindow(wm->display, win->app_window, win->width, win->height);
		//window_send_configure_event(win); // seems un-necessary
	}

	return win;
}

// Redraw the top bar only
void dialog_window_redraw(window *win)
{
	win->wm->current_theme->render_theme(win->wm->current_theme, win);
}

// Used during dynamic theme changing
void dialog_window_refresh_widgets(window *win)
{
	win->topbar_frame->redraw(win->topbar_frame);
	win->topbar_frame->show(win->topbar_frame);
}

// Lower this window
void dialog_window_hide(window *win)
{
	window_set_state(win, IconicState);
	win->is_mapped = False;
	XLowerWindow(win->wm->display, win->frame_window);

	// Lower its transients
	if(win->transient_next)
		win->transient_next->hide(win->transient_next);
}

// Show the window
void dialog_window_show(window *win)
{
	window_set_state(win, NormalState);
	
	win->is_mapped = True;
	XMapRaised(win->wm->display, win->frame_window);
	XMapSubwindows(win->wm->display, win->frame_window);

	// Raise the top frame... so that no app can stay on top of this
	win->topbar_frame->raise(win->topbar_frame);

	// Raise its transients
	if(win->transient_next)
		win->transient_next->show(win->transient_next);

	window_handle_focus(win);
}

// Kill this dialog
void dialog_window_destroy(window *win)
{
	window *mapped_win = win->wm->mapped_window;

	// First unlink yourself from the list of transients
	if(win->transient_prev) {
		win->transient_prev->transient_next = win->transient_next;
	}

	if(win->transient_next) {
		win->transient_next->transient_prev = win->transient_prev;
	}

	/*
	if(win->transient_next)
		window_handle_deletion(win->transient_next);
	*/

	// Is this required ??
	if(mapped_win)
		mapped_win->show(mapped_win);

	window_destroy(win);
}

// Set flags
void dialog_window_set_flags(window *win, unsigned long flags)
{
	win->flags.all_flags = flags;

	window_set_int_property(win, _AC_W_FLAGS, flags );
}

// Set flag
void dialog_window_set_flag(window *win, flag_num flag_no, Bool value)
{
	if(value)
		win->flags.all_flags |= (1<<flag_no);
	else
		win->flags.all_flags &= (~(1<<flag_no));

	window_set_int_property(win, _AC_W_FLAGS, win->flags.all_flags );
}

// Resize -- unlikely for dialogs
void dialog_window_set_size(window *win, unsigned long width, unsigned long height)
{
	int dialogbar_height = win->wm->current_theme->get_dialogbar_height(win->wm->current_theme);

	win->width=width;
	win->height=height;
	XResizeWindow(win->wm->display, win->app_window, width, height);
	XResizeWindow(win->wm->display, win->frame_window, width, height + dialogbar_height);
	window_send_configure_event(win);

	if(win->topbar_frame)
		win->topbar_frame->set_size(win->topbar_frame, win->width, dialogbar_height);
}

// Move -- when dragged
void dialog_window_set_position(window *win, int x, int y)
{
	int dialogbar_height = win->wm->current_theme->get_dialogbar_height(win->wm->current_theme);

	win->x=x;
	win->y=y;
	XMoveWindow(win->wm->display, win->frame_window, win->x, win->y - dialogbar_height);
}

// Dialog window click callback
Bool dialog_window_click_callback(px_widget *pw, px_event *pxe)
{
	window *win = (window *)pxe->private_data;

	// Raise dialog on click
	XRaiseWindow(win->wm->display, win->frame_window);

	return True;
}

// The following are positions
static int in_x=0, in_y=0;
static char flag=0;

// Dialog window release callback
Bool dialog_window_release_callback(px_widget *pw, px_event *pxe)
{
	flag = 0;
	return True;
}

// Dialog window motion callback
Bool dialog_window_motion_callback(px_widget *pw, px_event *pxe)
{
	int x,y;
	window *win = (window *)pxe->private_data;
	XMotionEvent xme = pxe->xev->xmotion;

	// Save the position of the pointer within the window
	if(!flag) {
		in_x = xme.x;
		in_y = xme.y;
		flag = 1;
	}

	// Calculate the window movement from the movement of the 
	x = xme.x_root-in_x;
	y = xme.y_root-in_y+25;

	win->set_position(win, x, y);

	return True;
}
